/****************************************************************
	Bookmarks
	by Naoyuki Tamura (tamura@kobe-u.ac.jp)
****************************************************************/

//
//	Page Class
//
function Page(href, title) {
	if (href == null)
		href = '';
	href = href.replace(/^\s+/, '').replace(/\s+$/, '')
		.replace(/\s/g, '%20').replace(/"/g, '%22')
		.replace(/</g, '%3c').replace(/>/g, '%3e');
	if (title == null)
		title = '';
	title = title.replace(/^\s+/, '').replace(/\s+$/, '')
		.replace(/\s/g, ' ');
	this.href = href;
	this.title = title;
	this.equal = PageEqual;
	this.toString = PageToString;
}

function PageEqual(page) {
	return this.href == page.href && this.title == page.title;
}

function PageToString(attr, t) {
	if (t == null)
		t = this.title;
	if (t == null || t == '')
		t = this.href;
	t = t.replace(/"/g, '&quot;')
		.replace(/</g, '&lt;').replace(/>/g, '&gt;');
	if (attr == null)
		attr = '';
	return '<a href="' + this.href + '"' + attr + '>' + t + '</a>';
}

//
//	BM Class
//
function BM(n, cn, cp, cd, ce) {
	this.pages = new Array(n);
	this.length = n;
	this.size = 0;
	this.cookieName = cn;
	this.cookiePath = cp;
	this.cookieDomain = cd;
	this.cookieExpire = ce;
	this.oldCookieValue = null;
	this.add = BMadd;
	this.remove = BMremove;
	this.find = BMfind;
	this.up = BMup;
	this.down = BMdown;
	this.load = BMload;
	this.save = BMsave;
	this.uptodate = BMuptodate;
}

function BMadd(page) {
	var i = this.find(page.href);
	if (i >= 0) {
		this.pages[i] = page;
		return i;
	}
	if (this.size >= this.length)
		return null;
	i = this.size;
	this.pages[i] = page;
	this.size++;
	return i;
}

function BMremove(i) {
	if (i < 0 || this.size <= i)
		return null;
	for (var j = i; j < this.size - 1; ++j)
		this.pages[j] = this.pages[j + 1];
	this.pages[this.size] = null;
	this.size--;
	return i;
}

function BMfind(href) {
	for (var i = 0; i < this.size; ++i) {
		if (this.pages[i].href == href)
			return i;
	}
	return -1;
}

function BMup(i) {
	var n = this.size;
	if (i < 0 || n <= i)
		return;
	var p = this.pages[i];
	var j = (i - 1 + n) % n;
	this.pages[i] = this.pages[j];
	this.pages[j] = p;
}

function BMdown(i) {
	var n = this.size;
	if (i < 0 || n <= i)
		return;
	var p = this.pages[i];
	var j = (i + 1) % n;
	this.pages[i] = this.pages[j];
	this.pages[j] = p;
}

function BMload() {
	var i;
	for (i = 0; i < this.length; ++i)
		this.pages[i] = null;
	this.size = 0;
	var cookie = getCookie(this.cookieName);
	this.oldCookieValue = cookie;
	if (cookie == null)
		return;
	i = cookie.lastIndexOf("\n");
	if (i >= 0)
		cookie = cookie.substring(0, i);
	var values = cookie.split("\n");
	for (i = 0; i < values.length && i < this.length; ++i) {
		var v = values[i].split("\t");
		this.pages[i] = new Page(v[0], v[1]);
	}
	this.size = i;
}

function BMsave() {
	var cookie = '';
	for (var i = 0; i < this.size; ++i) {
		var p = this.pages[i];
		if (p != null && p.href != null && p.title != null)
			cookie += p.href + "\t" + p.title + "\n";
	}
	var expire = this.cookieExpire;
	if (cookie == '')
		expire = -30;
	if (! setCookie(this.cookieName, cookie, this.cookiePath,
				this.cookieDomain, expire))
		return false;
	this.oldCookieValue = getCookie(this.cookieName);
	return true;
}

function BMuptodate() {
	return this.oldCookieValue == getCookie(this.cookieName);
}

function cookieSize(name) {
	if (document.cookie == null)
		return 0;
	name += '=';
	var p = document.cookie.split('; ');
	for (var i = 0; i < p.length; ++i) {
		if (p[i].indexOf(name) == 0)
			return p[i].length;
	}
	return 0;
}

function getCookie(name) {
	if (document.cookie == null)
		return null;
	name += '=';
	var p = document.cookie.split('; ');
	for (var i = 0; i < p.length; ++i) {
		if (p[i].indexOf(name) == 0)
			return unescape(p[i].substring(name.length));
	}
	return null;
}

function setCookie(name, cookie, path, domain, expDay) {
	cookie = name + '=' + escape(cookie);
	if (cookie.length > 4096) {
		alert('CookieΥ礭¸Ǥޤ');
		return false;
	}
	cookie += '; ';
	path = 'path=' + path + '; ';
	if (domain)
		domain = 'domain=' + domain + '; ';
	var d = new Date();
	d.setTime(d.getTime() + expDay*24*60*60*1000);
	document.cookie = cookie + path + domain +
		'expires=' + d.toGMTString() + ';';
	return true;
}

/****************************************************************
	Global Data and Shortcuts
****************************************************************/
var N = 50;			// Maximum number of bookmarks
var bmCookieName = 'bookmarks';
var bmCookiePath = '/';
var bmCookieDomain = '';
var bmCookieExpire = 365;		// days
var bm = new BM(N, bmCookieName, bmCookiePath,
			bmCookieDomain, bmCookieExpire);

//
//	For metcha.cgi
//
function refreshCheckMarks() {
	refreshCheck(document.results);
}

function refreshCheck(form) {
	if (bm.uptodate())
		return;
	bm.load();
	for (var i = 0; i < form.elements.length; ++i) {
		var cb = form.elements[i];
		if (cb == null || cb.type != 'checkbox')
			continue;
		cb.checked = false;
		if (cb.value == null || cb.value == '')
			continue;
		var k = cb.value.indexOf(' ');
		if (k < 0)
			continue;
		var href = cb.value.substring(0, k);
		if (bm.find(href) >= 0)
			cb.checked = true;
	}
}

function toBM(cb) {
	refreshCheck(cb.form);
	if (cb.value == null || cb.value == '')
		return;
	var k = cb.value.indexOf(' ');
	if (k < 0)
		return;
	var href = cb.value.substring(0, k);
	var title = cb.value.substring(k + 1);
	var i = bm.find(href);
	if (i >= 0) {
		if (! confirm('꤫ޤ?'))
			return;
		bm.remove(i);
		if (! bm.save()) {
			bm.load();
			return;
		}
		cb.checked = false;
	} else {
		if (bm.size == bm.length) {
			alert('ο¿ޤ');
			return;
		}
		if (! confirm('Ͽޤ?'))
			return;
		var p = new Page(href, title);
		if (bm.add(p) == null) {
			bm.load();
			return;
		}
		if (! bm.save()) {
			bm.load();
			return;
		}
		cb.checked = true;
	}
	if (parent != null && parent.BOOKMARKS != null) {
		var loc = parent.BOOKMARKS.location;
		loc.href = loc.href;
	}
	focus();
}

//
//	For bookmarks.html
//
function removeChecked(form) {
	var count = 0;
	for (var i = 0; i < form.elements.length; ++i) {
		var cb = form.elements[i];
		if (cb == null || cb.type != 'checkbox' || ! cb.checked)
			continue;
		if (cb.value == null || cb.value == '')
			continue;
		var j = bm.find(cb.value);
		if (j >= 0) {
			bm.remove(j);
			++count;
		}
	}
	if (count == 0) {
		alert('åΤĤڡޤ');
		return;
	}
	if (confirm(count + 'ڡޤ?')) {
		bm.save();
	}
	location.href = location.href;
}

function toggleChecked(form) {
	for (var i = 0; i < form.elements.length; ++i) {
		var c = form.elements[i];
		if (c.type == 'checkbox')
			c.checked = ! c.checked;
	}
}

//
//	For bmedit.html
//
function refreshEditor() {
	bm.load();
	var i = 0;
	for (var j = 0; j < document.forms.length; ++j) {
		var f = document.forms[j];
		if (f.bm == null)
			continue;
		f.bm.value = i;
		if (i < bm.size) {
			f.href.value = bm.pages[i].href;
			f.title.value = bm.pages[i].title;
			f.href.disabled = false;
			f.title.disabled = false;
		} else if (i == bm.size) {
			f.href.value = f.href.defaultValue;
			f.title.value = f.title.defaultValue;
			f.href.disabled = false;
			f.title.disabled = false;
		} else {
			f.href.value = 'Բ';
			f.title.value = 'Բ';
			f.href.disabled = true;
			f.title.disabled = true;
		}
		++i;
	}
	document.info.bmsize.value = bm.size;
	document.info.cookiesize.value = cookieSize(bmCookieName);
}

function submitEditor(form) {
	if (form.bm == null)
		return;
	var i = formIndex(form);
	if (i == null)
		return;
	var href = form.href.value;
	var title = form.title.value;
	if (href == '' || href == form.href.defaultValue) {
		alert('URLꤵƤޤ');
		return;
	}
	var p = new Page(href, title);
	href = p.href;
	title = p.title;
	if (0 <= i && i < bm.size) {
		if (p.equal(bm.pages[i]))
			return;
		var j = bm.find(href);
		if (j >= 0 && j != i) {
			alert('ƱڡϿߤǤ');
			return;
		}
		if (confirm('ѹޤ?')) {
			bm.pages[i] = p;
			bm.save();
			refreshEditor();
		}
	} else if (i == bm.size) {
		if (bm.find(href) >= 0) {
			alert('ƱڡϿߤǤ');
			return;
		}
		if (confirm('Ͽޤ?')) {
			if (bm.add(p) == null) {
				alert('ο¿ޤ');
			} else {
				bm.save();
				refreshEditor();
			}
		}
	} else {
		refreshEditor();
	}
}

function resetEditor(form) {
	if (form.bm == null)
		return;
	var i = formIndex(form);
	if (i == null | i < 0 || bm.size < i)
		return;
	if (i == bm.size) {
		form.href.value = form.href.defaultValue;
		form.title.value = form.title.defaultValue;
	} else {
		form.href.value = bm.pages[i].href;
		form.title.value = bm.pages[i].title;
	}
}

function removeEditor(form) {
	var i = formIndex(form);
	if (i == null | i < 0 || bm.size <= i)
		return;
	if (confirm('ޤ?')) {
		bm.remove(i);
		bm.save();
		refreshEditor();
	}
}

function upEditor(form) {
	var i = formIndex(form);
	if (i == null | i < 0 || bm.size <= i)
		return;
	bm.up(i);
	bm.save();
	refreshEditor();
}

function downEditor(form) {
	var i = formIndex(form);
	if (i == null | i < 0 || bm.size <= i)
		return;
	bm.down(i);
	bm.save();
	refreshEditor();
}

function formIndex(form) {
	if (form.bm == null || form.bm.value == null)
		return null;
	var i = parseInt(form.bm.value);
	if (isNaN(i))
		return null;
	return i;
}
